// ExamplePlugin.cpp : Defines the initialization routines for the DLL.
//

#include "ExamplePlugin.h"

#if defined(OS_WIN32)
#include <windows.h>
#include <malloc.h>
#endif

#include <stdio.h>
#include <string.h>
#include <stdlib.h>


// constructor of your plugin class
//
ExamplePlugin::ExamplePlugin()
{
}

// destructor of your plugin class
//
ExamplePlugin::~ExamplePlugin()
{
}

bool ExamplePlugin::initPlugin() // this function is called in GUI plugins once after class has been created - not called in pure computational plugins
{
	// This function call and check of return value is necessary to check whether access to QX works

	int ret = InitQXAccess();

	// Never continue, if the "InitQXAccess" function fails
	//
	if(!ret)
		return false;

	// Test whether the used QX version is new enough to support the commands used in your plugin
	//
	int Major, Minor, SubMinor;
	qxGetVersionOfBrainVoyagerQX(&Major, &Minor, &SubMinor);

	// We require v1.4.8
	if( !checkVersionGreaterEqualVersion(Major, Minor, SubMinor, 2, 1, 0) )
	{
		// we can use message box since this was available with the first QX version with plugin support (v1.3.8)
		qxMessageBox("Can not execute this plugin - BrainVoyager QX version 2.1 or higher required.");
		return false;
	}

	return true;
}

// THE MOST IMPORTANT EXPORTED FUNCTION
//
bool ExamplePlugin::execute()
{
	// This is the begin of the main plugin code
	//

	// This command makes sure that the user sees the "Log" tab. Use this tab to print useful information.
	qxShowLogTab();
	
	// This command sends a line of text to the "Log" tab.
	qxLogText(" ");
	qxLogText("// Plugin>  Running Example Plugin...");
	qxLogText("// Plugin>");

	char InfoString[501];
	// Test access to BVQX and plugins directory:
	char PathToBVQX[301];
	qxGetBrainVoyagerQXPath(PathToBVQX);
	sprintf(InfoString, "// Plugin> Path to BVQX: %s", PathToBVQX);
	qxLogText(InfoString);

	char PathToPlugins[301];
	qxGetPluginsPath(PathToPlugins);
	sprintf(InfoString, "// Plugin> Path to Plugins: %s", PathToPlugins);
	qxLogText(InfoString);
	qxLogText("// Plugin>");

	// Is there a document in the workspace?
	int NDocs = qxGetNumberOfDocsInWorkspace();
	sprintf(InfoString, "// Plugin> Number of documents: %i", NDocs);
	qxLogText(InfoString);
	if(NDocs == 0)
	{
		// You can use simple dialogs to present text to the user or to get user input. Here we present an error message.
		qxMessageBox("This plugin requires that a VMR document is available and selected in BrainVoyager QX.");
		return false;
	}

	// Optional - Cycle through all documents to learn about the document's type and name
	int i, DocType;
	char DocTypeName[4], DocFullFileName[301];
	for(i=0; i<NDocs; i++)
	{
		DocType = qxGetDocumentType(i); // 1 -> FMR, 2 -> DMR, 3 -> VMR, 4 -> SRF
		strcpy(DocTypeName, qxGetDocumentTypeName(i));

		sprintf(InfoString, "// Plugin> Document %i is a %s (type-id: %i)", (i+1), DocTypeName, DocType);
		qxLogText(InfoString);
	}

	int current_doc_index = qxGetIDOfCurrentDocument(); // we know that we have at least one doc from code above, otherwise check for return value < 0
	DocType = qxGetDocumentType(current_doc_index);
	if(DocType != 3)
	{
		qxMessageBox("The currently active document is not a VMR document.");
		return false;
	}
	
	// Most qx-API commands work on the "current" document in the workspace
	// Normally the user will have the relevant document selected before launching a plugin (the relevant document might often be the only one in the worksspace and is then automatically selected)
	// If your plugin needs to use the data from more than one document, it is, however, important to get access to several data sets
	// With the following command, you can pro programmatically select any document in the workspace as the "current" one (you can check the type of a doc with the "qxGetDocumentType" command as shown above)
	// qxSetAsCurrentDocument(0);

	// Get a pointer to the "current" (i.e. "active", "selected") VMR
	int DimX, DimY, DimZ;
	unsigned char *VMR = qxGetCurrentVMR(&DimX, &DimY, &DimZ);

	if(VMR == 0) // if the returned pointer is "0", the active document is not a VMR, or there is no document at all available in QX
	{
		qxMessageBox("This plugin requires that a VMR document is available and selected in BrainVoyager QX.");
		qxLogText( "// Plugin>  Could not get VMR!" );
		return false;
	}

	// Here we use a simple "Yes/No" dialog to ask the user whether he wants to continue
	int ret = qxYesNoMessageBox("This plugin inverts the intensities of the current VMR\n\nDo you want to continue?");
	if(ret == 0)
		return false;

	// Print the dimensions of the current VMR to the "Log"
	sprintf(InfoString, "// Plugin>  DimX: %i  DimY: %i  DimZ: %i", DimX, DimY, DimZ);
	qxLogText( InfoString );

	// Now invert the intensities of the VMR

	int x, y, z;
	unsigned char val;

	for(z=0; z<DimZ; z++)
	{
		for(y=0; y<DimY; y++)
		{
			for(x=0; x<DimX; x++)
			{
				val = VMR[z*DimX*DimY + y*DimX + x];
				if(val < 30)
					VMR[z*DimX*DimY + y*DimX + x] = 0;
				else
					VMR[z*DimX*DimY + y*DimX + x] = 225 - val;
			}
		}
	}

	// To make the change visible, update the active window in QX
	qxUpdateActiveWindow();

	qxLogText("// Plugin>  Example Plugin completed!");
	qxLogText(" ");

	return true;
}


//
// C access functions (extern "C" in header)
//

// THIS FUNCTION CREATES YOUR PLUGIN CLASS
//
PLUGIN_ACCESS BVQXPluginInterface *createPlugin() 
{
	return new ExamplePlugin();
}

// return a name for your plugin
//
PLUGIN_ACCESS const char *getPluginName() 
{
	return "Example Plugin";
}

// returns a descriptive string for your plugin shown in the QX "Descriptions of Plugins" window
//   it is recommended to use a static C string to describe your plugin
//   (the potentially lengthy string needs then not be created repeatedly in successive calls)
//   Note that you may use simple HTML tags to format the string, such as <b></b>, <i></i> and <br>
//
static const char *PluginDescription = "This simple plugin inverts the intensity values of the current VMR. Use its source code to get started with writing your own plugins.<br>";

PLUGIN_ACCESS const char *getPluginDescription() 
{
	return PluginDescription;
}

// get credit for your work!
//
PLUGIN_ACCESS const char *getAuthor()
{
	return "<i>Rainer Goebel</i>, Brain Innovation B.V.";
}

// you may provide the name for a HTML file containing online help
//
PLUGIN_ACCESS const char *getHelpFile()
{
	return "ExamplePluginHelp/index.html";
}

// you may provide a version string ("Major.Minor") for (updated) releases of your plugins
//
PLUGIN_ACCESS const char *getPluginVersion()
{
	return "2.0";
}

// NEW: If you want to use GUI dialogs, return a GUI script file, if not, return empty C string
//
PLUGIN_ACCESS const char *getGUIScript()
{
	return "";
}


